package Apache::Admin::Config::Expand::include;
use strict;
use warnings;
use Apache::Admin::Config;
use Apache::Admin::Config::Expand;
use File::Spec;
use Carp;

sub new {
    bless { included => {} }, shift
}

sub expand {
    my ($self, $tree, $d, $repl) = @_;

    if ($d->type eq 'directive' && $d->name =~ /^include(optional)?$/i) {
	my $optional = $1;

	my $pat = $tree->dequote($d->value);
	unless (File::Spec->file_name_is_absolute($pat)) {
	    if (my $d = $tree->server_root) {
		$pat = File::Spec->catfile($d, $pat);
	    }
	}

	my @filelist = glob $pat;
	if (@filelist) {
	    foreach my $file (@filelist) {
		if ($self->check_included($file)) {
		    croak "file $file already included";
		}
		if (my $inc = new Apache::Admin::Config($file,
							@{$tree->options})) {
		    push @$repl, $inc->select;
		} else {
		    croak $Apache::Admin::Config::ERROR;
		}
	    }
	}
	return 1;
    }
	    
    return 0;
}

sub check_included {
    my ($self, $file) = @_;
    my ($dev,$ino) = stat($file) or return 0;
    return 1 if $self->{included}{$dev}{$ino};
    $self->{included}{$dev}{$ino} = 1;
    return 0;
}

1;
